TOP_ITEM_NOBORDER = 0
TOP_ITEM_ARROW = 1
TOP_ITEM_LEFT = 2
TOP_ITEM_RIGHT = 3

ROW_NAME = 1
ROW_PLATFORM = 3

TOP_WIDTH = 64

COVER_POS = 0.75
# TOP_HEIGHT = 1.0 / 8.0
# TOP_HEIGHT = 60

# actual/visible height is 60, height incl shadow is 90
# - not using shadow anymore
# TOP_HEIGHT = 90
TOP_HEIGHT = 60
TOP_HEIGHT_VISIBLE = 60
