# this is a quick hack tp fix problems with some modules not being imported
# by py2exe / py2app otherwise

# import workspace.apps.adf_creator_app
# import workspace.apps.hdf_creator_app
# import workspace.apps.joystick_config_app
# import workspace.apps.locker_uploader
# import workspace.apps.login
# import workspace.apps.logout
# import workspace.apps.refresh

# import workspace.apps.clock_app
# import workspace.prefs.audio
# import workspace.prefs.experimental_features
# import workspace.prefs.filter
# import workspace.prefs.game_database
# import workspace.prefs.joystick
# import workspace.prefs.keyboard
# import workspace.prefs.language
# import workspace.prefs.mouse
# import workspace.prefs.netplay
# # import workspace.prefs.opengl
# import workspace.prefs.scan
# import workspace.prefs.video
