PACKAGE_COMMIT=
PACKAGE_DISPLAY_NAME="FS-UAE Launcher"
PACKAGE_MACOS_BUNDLE_ID=no.fengestad.fs-uae-launcher
PACKAGE_NAME=fs-uae-launcher
PACKAGE_NAME_PRETTY=FS-UAE-Launcher
PACKAGE_TYPE=fs-app-plugin
PACKAGE_VERSION=3.2.35
PACKAGE_VERSION_MAJOR=3
PACKAGE_VERSION_MINOR=2
PACKAGE_VERSION_REVISION=35
PACKAGE_VERSION_TAG=
